<?php

use Connectix\Components\IntegrationHook;
use ctx\traits\LibraryLoader;
use ctx\traits\StoreLoader;

class ControllerExtensionModuleConnectix extends Controller
{
    use StoreLoader;
    use LibraryLoader;

    private $error = array();
    private $store_id = 0;
    private $store_url;
    private $endpoint_url;

    public function __construct($registry)
    {
        parent::__construct($registry);

        $this->store_id = isset($this->request->get['store_id']) ? (int)$this->request->get['store_id'] : 0;

        $this->loadStore($this->store_id);
        $this->loadLibrary($this->store_id);

        if ($this->store_id === 0) {
            $this->store_url = basename(DIR_TEMPLATE) == 'template' ? HTTPS_CATALOG : HTTPS_SERVER;
        } else {
            $this->load->model('setting/setting');
            $this->store_url = $this->model_setting_setting->getSettingValue('config_ssl', $this->store_id);
        }

        $this->endpoint_url = rtrim($this->store_url, '/').'/index.php?route=%s';

        $settings = $this->model_setting_setting->getSetting('module_connectix');

        if (array_key_exists('module_connectix_pluginToken', $settings)) {
            $settings['module_connectix_integrationToken'] = $settings['module_connectix_pluginToken'];
            unset($settings['module_connectix_pluginToken']);
            $this->model_setting_setting->editSetting('module_connectix', $settings, $this->store_id);
        }
    }

    public function index()
    {
        $this->load->language('extension/module/connectix');
        $this->load->model('extension/module/connectix');

        // explicitly call createTables for the new releases
        $this->model_extension_module_connectix->createTables();

        if (isset($this->request->post['ajax'])) {
            $library = $this->registry->get('connectix');
            $this->response->addHeader('Content-Type: application/json');
            $data = '';
            if (isset($this->request->post['action'])) {
                switch ($this->request->post['action']) {
                    case 'getData':
                        $data = [
                            'settings' => $this->getSettings(),
                            'orderStatuses' => $this->fetchOrderStatuses(),
                            'templates' => $this->model_extension_module_connectix->fetchTemplates(),
                            'rules' => $library->fetchRules(),
                        ];
                        break;
                    case 'saveSettings':
                        $templates = null;

                        if ('POST' === $this->request->server['REQUEST_METHOD'] && $this->validatePermission()) {
                            $postData = array();
                            $oldSettings = $this->getSettings(false, true);
                            foreach (array_keys($oldSettings) as $setting) {
                                $name = $setting;
                                if ('token' === $setting) {
                                    $name = 'connectixToken';
                                }

                                $postData[$setting] = $this->getSettingValue($name, $oldSettings[$setting]);
                                if ('enabled' === $setting) {
                                    $postData['status'] = $postData[$setting];
                                }
                            }

                            if (!empty($postData)) {
                                $this->applyNewSettings($postData);
                            }

                            $newSettings = $this->getSettings(false, true);

                            $isTokenChanged = $oldSettings['token'] !== $newSettings['token'];
                            if ($isTokenChanged && $oldSettings['token'] && $oldSettings['integrationToken']) {
                                $library->unregister($this->getSettingValue('token'), [
                                    'system' => $oldSettings['system'],
                                    'token' => $oldSettings['integrationToken'],
                                    'secret' => $oldSettings['secret'],
                                ]);
                                $postData['integrationToken'] = null;
                            }

                            $templates = null;
                            if ($isTokenChanged && $newSettings['token']) {
                                $response = $library->register($newSettings['token'], [
                                    'system' => $newSettings['system'],
                                    'type' => 'ecommerce',
                                    'platform' => 'OpenCart',
                                    'secret' => $newSettings['secret'],
                                ]);

                                if ($response) {
                                    $postData['integrationToken'] = $response['token'];
                                    $templates = $library->syncTemplates();
                                }
                            }

                            if (!empty($postData)) {
                                $this->applyNewSettings($postData);
                                $newSettings = $this->getSettings(false, true);
                            }
                            $this->unregisterHooks($oldSettings, $newSettings);
                            $this->registerHooks($oldSettings, $newSettings);
                        }

                        $data = $this->getSettings(false);
                        if ($templates) {
                            $data['templates'] = $templates;
                        }
                        break;
                    case 'syncTemplates':
                        $data = $library->syncTemplates();
                        break;
                    case 'saveRule':
                        if ('POST' === $this->request->server['REQUEST_METHOD'] && $this->validatePermission()) {
                            $data = $this->model_extension_module_connectix->saveRule($this->request->post);
                        }
                        break;
                    case 'deleteRule':
                        if ('POST' === $this->request->server['REQUEST_METHOD'] && $this->validatePermission()) {
                            $data = $this->model_extension_module_connectix->deleteRule($this->request->post['id']);
                        }
                        break;
                    default:
                        $data = [];
                        break;
                }
            }

            $this->response->setOutput(json_encode($data));
            return;
        }

        $this->document->setTitle($this->language->get('heading_title'));

        if (isset($this->error['warning'])) {
            $data['error_warning'] = $this->error['warning'];
        } else {
            $data['error_warning'] = '';
        }

        if (isset($this->error['token'])) {
            $data['error_token'] = $this->error['token'];
        } else {
            $data['error_token'] = '';
        }

        $data['breadcrumbs'] = array();
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', 'user_token='.$this->session->data['user_token'], true),
        );
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_extensions'),
            'href' => $this->url->link(
                'marketplace/extension',
                'user_token='.$this->session->data['user_token'].'&type=module',
                true
            ),
        );
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link(
                'extension/module/connectix',
                'user_token='.$this->session->data['user_token'],
                true
            ),
        );


        // Translations
        $data['ask_title'] = $this->language->get('ask_title');
        $data['ask_button_cancel'] = $this->language->get('ask_button_cancel');
        $data['ask_button_confirm'] = $this->language->get('ask_button_confirm');

        $data['settings_title'] = $this->language->get('settings_title');
        $data['settings_label_enabled'] = $this->language->get('settings_label_enabled');
        $data['settings_label_token'] = $this->language->get('settings_label_token');
        $data['settings_label_token_help'] = $this->language->get('settings_label_token_help');
        $data['settings_label_system'] = $this->language->get('settings_label_system');
        $data['settings_label_system_sandbox'] = $this->language->get('settings_label_system_sandbox');
        $data['settings_label_system_live'] = $this->language->get('settings_label_system_live');
        $data['settings_label_sandboxUrl'] = $this->language->get('settings_label_sandboxUrl');
        $data['settings_label_sandboxUrl_help'] = $this->language->get('settings_label_sandboxUrl_help');
        $data['settings_delivered_status'] = $this->language->get('settings_label_delivered_status');
        $data['settings_delivered_status_help'] = $this->language->get('settings_label_delivered_status_help');
        $data['settings_add_contacts'] = $this->language->get('settings_add_contacts');
        $data['settings_track_delivery'] = $this->language->get('settings_track_delivery');
        $data['settings_label_show_cron'] = $this->language->get('settings_label_show_cron');
        $data['settings_label_hide_cron'] = $this->language->get('settings_label_hide_cron');
        $data['settings_label_save'] = $this->language->get('settings_label_save');
        $data['settings_token_placeholder'] = $this->language->get('settings_token_placeholder');
        $data['settings_token_already_entered'] = $this->language->get('settings_token_already_entered');
        $data['settings_sandboxUrl_placeholder'] = $this->language->get('settings_sandboxUrl_placeholder');
        $data['settings_label_confirmOrder'] = $this->language->get('settings_label_confirmOrder');
        $data['settings_label_confirmedOrderStatus'] = $this->language->get('settings_label_confirmedOrderStatus');
        $data['settings_label_confirmedOrderStatus_help'] = $this->language->get('settings_label_confirmedOrderStatus_help');
        $data['settings_label_confirmedOrderUrl'] = $this->language->get('settings_label_confirmedOrderUrl');
        $data['settings_label_confirmedOrderUrl_help'] = $this->language->get('settings_label_confirmedOrderUrl_help');
        $data['settings_label_cancelOrder'] = $this->language->get('settings_label_cancelOrder');
        $data['settings_label_canceledOrderStatus'] = $this->language->get('settings_label_canceledOrderStatus');
        $data['settings_label_canceledOrderStatus_help'] = $this->language->get('settings_label_canceledOrderStatus_help');
        $data['settings_label_canceledOrderUrl'] = $this->language->get('settings_label_canceledOrderUrl');
        $data['settings_label_canceledOrderUrl_help'] = $this->language->get('settings_label_canceledOrderUrl_help');
        $data['settings_label_confirmationUrlsValidity'] = $this->language->get('settings_label_confirmationUrlsValidity');
        $data['settings_label_confirmationUrlsValidity_help'] = $this->language->get('settings_label_confirmationUrlsValidity_help');
        $data['settings_label_confirmationUrlsExpiredUrl'] = $this->language->get('settings_label_confirmationUrlsExpiredUrl');
        $data['settings_label_confirmationUrlsExpiredUrl_help'] = $this->language->get('settings_label_confirmationUrlsExpiredUrl_help');

        $data['templates_button'] = $this->language->get('templates_button');
        $data['templates_no_templates'] = $this->language->get('templates_no_templates');
        $data['templates_status_pending'] = $this->language->get('templates_status_pending');
        $data['templates_status_rejected'] = $this->language->get('templates_status_rejected');
        $data['templates_status_waiting_information'] = $this->language->get('templates_status_waiting_information');
        $data['templates_status_approved'] = $this->language->get('templates_status_approved');

        $data['rules_adding_title'] = $this->language->get('rules_adding_title');
        $data['rules_editing_title'] = $this->language->get('rules_editing_title');
        $data['rules_button'] = $this->language->get('rules_button');
        $data['rules_no_rules'] = $this->language->get('rules_no_rules');
        $data['rules_label_enabled'] = $this->language->get('rules_label_enabled');
        $data['rules_label_not_enabled'] = $this->language->get('rules_label_not_enabled');
        $data['rules_label_fallback'] = $this->language->get('rules_label_fallback');
        $data['rules_label_type'] = $this->language->get('rules_label_type');
        $data['rules_label_delivery_type'] = $this->language->get('rules_label_delivery_type');
        $data['rules_label_order_status'] = $this->language->get('rules_label_order_status');
        $data['rules_label_add_template'] = $this->language->get('rules_label_add_template');
        $data['rules_label_add_another_template'] = $this->language->get('rules_label_add_another_template');
        $data['rules_label_days_not_picked_up'] = $this->language->get('rules_label_days_not_picked_up');
        $data['rules_label_cancel'] = $this->language->get('rules_label_cancel');
        $data['rules_label_save'] = $this->language->get('rules_label_save');
        $data['rules_days'] = $this->language->get('rules_days');
        $data['rules_order_status_placeholder'] = $this->language->get('rules_order_status_placeholder');
        $data['rules_templates_placeholder'] = $this->language->get('rules_templates_placeholder');
        $data['rules_delivery_type_placeholder'] = $this->language->get('rules_delivery_type_placeholder');
        $data['rules_delivery_type_door'] = $this->language->get('rules_delivery_type_door');
        $data['rules_delivery_type_office'] = $this->language->get('rules_delivery_type_office');
        $data['rules_channel'] = $this->language->get('rules_channel');
        $data['rules_template'] = $this->language->get('rules_template');
        $data['rules_actions'] = $this->language->get('rules_actions');
        $data['rules_type_welcome'] = $this->language->get('rules_type_welcome');
        $data['rules_type_change_order_status'] = $this->language->get('rules_type_change_order_status');
        $data['rules_type_change_delivery_status'] = $this->language->get('rules_type_change_delivery_status');
        $data['rules_type_change_not_picked_up'] = $this->language->get('rules_type_change_not_picked_up');
        $data['rules_type_placeholder'] = $this->language->get('rules_type_placeholder');
        $data['rules_status_pending'] = $this->language->get('rules_status_pending');
        $data['rules_status_rejected'] = $this->language->get('rules_status_rejected');
        $data['rules_status_waiting_information'] = $this->language->get('rules_status_waiting_information');
        $data['rules_status_approved'] = $this->language->get('rules_status_approved');

        $data['cancel'] = $this->url->link(
            'marketplace/extension',
            'user_token='.$this->session->data['user_token'].'&type=module',
            true
        );
        $data['link'] = str_replace(
            '&amp;',
            '&',
            $this->url->link(
                'extension/module/connectix',
                'store_id='.$this->store_id.'&user_token='.$this->session->data['user_token'],
                true
            )
        );

        $uiFile = DIR_APPLICATION.'view/javascript/connectix/ui/asset-manifest.json';
        if (file_exists($uiFile)) {
            $ui = json_decode(file_get_contents($uiFile), true);
            if (isset($ui['entrypoints']) && !empty($ui['entrypoints'])) {
                foreach ($ui['entrypoints'] as $entrypoints) {
                    if (strstr($entrypoints, 'css')) {
                        $this->document->addStyle('view/javascript/connectix/ui/'.$entrypoints);
                    } elseif (strstr($entrypoints, 'runtime')) {
                        $this->document->addScript('view/javascript/connectix/ui/'.$entrypoints, 'footer');
                    } else {
                        $this->document->addScript('view/javascript/connectix/ui/'.$entrypoints);
                    }
                }
            }

        }

        $data['scripts'] = $this->document->getScripts('footer');
        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view('extension/module/connectix', $data));
    }

    /**
     * @param array $oldSettings
     * @param array $newSettings
     */
    private function unregisterHooks(array $oldSettings, array $newSettings)
    {
        $library = $this->registry->get('connectix');
        $shouldUnregister = $oldSettings['token'] !== $newSettings['token'] || ($oldSettings['enabled'] !== $newSettings['enabled'] && !$newSettings['enabled']);

        $postData = [];
        foreach ([IntegrationHook::TYPE_CALLBACK, IntegrationHook::TYPE_INBOUND, IntegrationHook::TYPE_TEMPLATE, IntegrationHook::TYPE_TRACKING] as $type) {
            $shouldUnregisterTrackingHook = $type === IntegrationHook::TYPE_TRACKING && $oldSettings['trackDelivery'] !== $newSettings['trackDelivery'] && !$newSettings['trackDelivery'];
            if ($shouldUnregister || $shouldUnregisterTrackingHook) {
                $hookSettingsName = 'hook'.ucfirst($type);
                $hookId = $newSettings[$hookSettingsName];
                if (!empty($hookId) && $oldSettings['token']) {
                    $library->unregisterHook($oldSettings['token'], [
                        'system' => $newSettings['system'],
                        'id' => $hookId,
                    ]);
                    $postData[$hookSettingsName] = null;
                }
            }
        }

        if (!empty($postData)) {
            $this->applyNewSettings($postData);
        }
    }

    /**
     * @param array $oldSettings
     * @param array $newSettings
     */
    private function registerHooks(array $oldSettings, array $newSettings)
    {
        // Hooks callback|inbound|template|tracking
        $library = $this->registry->get('connectix');
        $token = $newSettings['token'];
        $secret = $newSettings['secret'];
        $integrationToken = $newSettings['integrationToken'];

        $shouldRegister = $integrationToken && (($oldSettings['enabled'] !== $newSettings['enabled'] && $newSettings['enabled']) || ($oldSettings['token'] !== $token && $token && $newSettings['enabled']));

        $postData = [];
        foreach ([IntegrationHook::TYPE_CALLBACK, IntegrationHook::TYPE_INBOUND, IntegrationHook::TYPE_TEMPLATE, IntegrationHook::TYPE_TRACKING] as $type) {
            $shouldRegisterTrackingHook = $integrationToken && $newSettings['enabled'] && $type === IntegrationHook::TYPE_TRACKING && $oldSettings['trackDelivery'] !== $newSettings['trackDelivery'] && $newSettings['trackDelivery'];
            if (($type !== IntegrationHook::TYPE_TRACKING && $shouldRegister) || $shouldRegisterTrackingHook) {
                $hookSettingsName = 'hook'.ucfirst($type);
                $data = [
                    'system' => $newSettings['system'],
                    'token' => $newSettings['integrationToken'],
                    'type' => $type,
                    'url' => sprintf($this->endpoint_url, 'extension/connectix/api&store_id='.$this->store_id.'&type='.$type.'&secret='.$secret),
                ];

                if ($response = $library->registerHook($token, $data) ) {
                    $postData[$hookSettingsName] = $response['id'];
                }
            }
        }

        if (!empty($postData)) {
            $this->applyNewSettings($postData);
        }
    }

    public function install()
    {
        $this->load->model('extension/module/connectix');
        $this->applyNewSettings([
            'status' => 0, // Shows Extension list status
            'enabled' => 0,
            'token' => null,
            'system' => 'sandbox',
            'sandboxUrl' => null,
            'secret' => md5(uniqid((string)mt_rand(), true)),
            'addContacts' => 1,
            'trackDelivery' => 1,
            'integrationToken' => null,
            'confirmOrder' => 0,
            'confirmedOrderStatus' => null,
            'confirmedOrderUrl' => null,
            'cancelOrder' => 0,
            'canceledOrderStatus' => null,
            'canceledOrderUrl' => null,
            'confirmationUrlsValidity' => null,
            'confirmationUrlsExpiredUrl' => null,
            'hookCallback' => null,
            'hookInbound' => null,
            'hookTemplate' => null,
            'hookTracking' => null,
        ]);
        $this->model_extension_module_connectix->createTables();
        $this->model_extension_module_connectix->createIndexes();
        $this->model_extension_module_connectix->createEvents();
    }

    public function uninstall()
    {
        $library = $this->registry->get('connectix');
        $library->unregister($this->getSettingValue('token'), [
            'system' => $this->getSettingValue('system', 'sandbox'),
            'token' => $this->getSettingValue('integrationToken'),
            'secret' => $this->getSettingValue('secret')
        ]);
        $this->load->model('extension/module/connectix');
        $this->model_extension_module_connectix->dropTables();
        $this->model_extension_module_connectix->deleteEvents();
    }

    ##########################################################
    #                     Helper Methods                     #
    ##########################################################
    /**
     * @param string $value
     * @return string|null
     */
    private function nullableValue($value)
    {
        if ('' === $value) {
            return null;
        }

        return $value;
    }

    protected function applyNewSettings($new_settings)
    {
        $this->load->model('setting/setting');
        $old_settings = $this->model_setting_setting->getSetting('module_connectix', $this->store_id);
        $old_settings_keys = array_map(
            function ($setting) {
                return str_replace('module_connectix_', '', $setting);
            },
            array_keys($old_settings)
        );
        $old_settings = array_combine($old_settings_keys, $old_settings);

        $new_settings = array_merge($old_settings, $new_settings);

        $post_settings = array_map(
            function ($setting) {
                return 'module_connectix_'.$setting;
            },
            array_keys($new_settings)
        );

        $this->model_setting_setting->editSetting(
            'module_connectix',
            array_combine($post_settings, $new_settings),
            $this->store_id
        );

        foreach ($new_settings as $key => $value) {
            $this->setting->set($key, $value);
        }
    }

    /**
     * @param $key
     * @param null $default
     * @param bool $useRequest
     * @return mixed|null
     */
    protected function getSettingValue($key, $default = null, $useRequest = true)
    {
        if ($useRequest && isset($this->request->post[$key])) {
            return $this->request->post[$key];
        } else {
            if ($this->setting->has($key)) {
                return $this->setting->get($key);
            } else {
                return $default;
            }
        }
    }

    /**
     * @param bool $useRequest
     * @param bool $full
     * @return array
     */
    private function getSettings($useRequest = true, $full = false)
    {
        $settings = [
            'enabled' => (bool)$this->getSettingValue('enabled', false, $useRequest),
            'token' => $this->getSettingValue('token', null, $useRequest),
            'system' => $this->getSettingValue('system', null, $useRequest),
            'sandboxUrl' => $this->getSettingValue('sandboxUrl', null, $useRequest),
            'addContacts' => (bool)$this->getSettingValue('addContacts', true, $useRequest),
            'trackDelivery' => (bool)$this->getSettingValue('trackDelivery', true, $useRequest),
            'deliveredStatus' => $this->getSettingValue('deliveredStatus', null, $useRequest),
            'confirmOrder' => (bool)$this->getSettingValue('confirmOrder', null, $useRequest),
            'confirmedOrderStatus' => $this->getSettingValue('confirmedOrderStatus', null, $useRequest),
            'confirmedOrderUrl' => $this->getSettingValue('confirmedOrderUrl', null, $useRequest),
            'cancelOrder' => (bool)$this->getSettingValue('cancelOrder', null, $useRequest),
            'canceledOrderStatus' => $this->getSettingValue('canceledOrderStatus', null, $useRequest),
            'canceledOrderUrl' => $this->getSettingValue('canceledOrderUrl', null, $useRequest),
            'confirmationUrlsValidity' => $this->getSettingValue('confirmationUrlsValidity', null, $useRequest),
            'confirmationUrlsExpiredUrl' => $this->getSettingValue('confirmationUrlsExpiredUrl', null, $useRequest),
        ];

        if ($full) {
            $settings['secret'] = $this->getSettingValue('secret', null, false);
            $settings['integrationToken'] = $this->getSettingValue('integrationToken', null, false);
            $settings['hookCallback'] = $this->getSettingValue('hookCallback', null, false);
            $settings['hookInbound'] = $this->getSettingValue('hookInbound', null, false);
            $settings['hookTemplate'] = $this->getSettingValue('hookTemplate', null, false);
            $settings['hookTracking'] = $this->getSettingValue('hookTracking', null, false);
        }

        return $settings;
    }

    /**
     * @return array|array[]
     */
    private function fetchOrderStatuses()
    {
        $this->load->model('localisation/order_status');
        $order_statuses = $this->model_localisation_order_status->getOrderStatuses();

        return array_map(
            function ($order_status) {
                return [
                    'key' => $order_status['order_status_id'],
                    'value' => $order_status['order_status_id'],
                    'text' => $order_status['name'],
                ];
            },
            $order_statuses
        );
    }

    /**
     * @return bool
     */
    protected function validatePermission()
    {
        if (!$this->user->hasPermission('modify', 'extension/module/connectix')) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        return !$this->error;
    }

    public function addTrackingNumber()
    {
        $this->load->language('extension/module/connectix');

        $json = array();

        if (!$this->user->hasPermission('modify', 'extension/module/connectix')) {
            $json['error'] = $this->language->get('error_permission');
        } else {
            if (isset($this->request->get['order_id'])) {
                $order_id = $this->request->get['order_id'];
            } else {
                $order_id = 0;
            }

            if (isset($this->request->post['tracking_number'])) {
                $tracking_number = $this->request->post['tracking_number'];
            } else {
                $tracking_number = '';
            }

            $this->load->model('sale/order');
            $order_info = $this->model_sale_order->getOrder($order_id);
            if ($order_info) {
                $this->load->model('extension/module/connectix');
                $this->model_extension_module_connectix->addTrackingNumber($order_id, $tracking_number);
            }

            $json['success'] = $this->language->get('text_tracking_number_added');
        }
        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($json));
    }
}
