<?php

namespace ctx\traits;

use ctx\ConnectixLib;

trait Db
{
    /**
     * @param string|null $type
     * @param string|int|null $id
     * @return array
     */
    public function fetchRules($type = null, $id = null)
    {
        $where = [];
        if ($type && in_array(
                $type,
                [
                    ConnectixLib::TYPE_NEW_ORDER,
                    ConnectixLib::TYPE_CHANGE_ORDER_STATUS,
                    ConnectixLib::TYPE_READY_FOR_DELIVERY,
                    ConnectixLib::TYPE_NOT_PICKED_UP,
                ]
            )) {
            $where[] = 'r.type = "'.$this->db->escape($type).'"';
        }
        if ($id) {
            $where[] = 'r.id = "'.$this->db->escape($id).'"';
        }
        if (!empty($where) && !$id) {
            $where[] = 'r.enabled = "1"';
        }

        $rules = $this->db->query(
            'SELECT
                    r.*,
                    rt.position as rt_position,
                    t.id as t_id,
                    t.name as t_name,
                    t.channel as t_channel,
                    t.status as t_status,
                    t.provider_id as t_provider_id
                FROM
                    '.DB_PREFIX.'connectix_rules r
                LEFT JOIN '.DB_PREFIX.'connectix_rules_templates rt ON rt.rule_id = r.id
                LEFT JOIN '.DB_PREFIX.'connectix_templates t ON rt.template_id = t.id
                '.(!empty($where) ? 'WHERE '.implode(' AND ', $where) : '').'
                ORDER BY rt.position ASC'
        )->rows;

        if (!$rules) {
            return [];
        }

        $normalized = [];
        foreach ($rules as $rule) {
            $template = [
                'id' => $rule['t_id'],
                'position' => $rule['rt_position'],
                'name' => $rule['t_name'],
                'channel' => $rule['t_channel'],
                'status' => $rule['t_status'],
                'provider_id' => $rule['t_provider_id'],
            ];

            if (!isset($normalized[$rule['id']])) {
                $normalized[$rule['id']] = [
                    'id' => $rule['id'],
                    'fallback' => $rule['fallback'] === '1',
                    'type' => $rule['type'] !== '' ? $rule['type'] : null,
                    'order_status' => $rule['order_status'] !== '' ? $rule['order_status'] : null,
                    'delivery_type' => $rule['delivery_type'] !== '' ? $rule['delivery_type'] : null,
                    'days_not_picked_up' => $rule['days_not_picked_up'] !== '' ? $rule['days_not_picked_up'] : null,
                    'enabled' => '1' === $rule['enabled'],
                    'templates' => [$template],
                ];
            } else {
                $normalized[$rule['id']]['templates'][] = $template;
            }
        }

        return array_values($normalized);
    }

    /**
     * @param array $data
     */
    public function writeMessage($data)
    {
        $sqlData = $this->prepareDataForPersist($data);
        if (!$sqlData) {
            return;
        }
        $this->db->query(
            'INSERT INTO '.DB_PREFIX.'connectix_messages ('.$sqlData['columns'].') VALUES ('.$sqlData['values'].')'
        );
    }

    /**
     * @param array $data
     * @return string
     */
    public function prepareDataForUpdate($data)
    {
        $output = array();
        foreach ($data as $name => $value) {
            $value = $this->db->escape($value);
            $output[] = "`{$name}` = '{$value}'";
        }

        return implode(', ', $output);
    }

    /**
     * @param array $data
     * @return array[]
     */
    public function prepareDataForPersist($data)
    {
        if (empty($data)) {
            return null;
        }

        $columns = $values = array();
        foreach ($data as $name => $value) {
            $columns[] = "`$name`";
            $values[] = "'".$this->db->escape($value)."'";
        }

        return [
            'columns' => implode(', ', $columns),
            'values' => implode(', ', $values),
        ];
    }
}
